@@ -18,6 +18,14 @@
 #'   \code{facets}.
 #' @param add.data,rawdata Logical, if \code{TRUE}, a layer with raw data from response by
 #'   predictor on the x-axis, plotted as point-geoms, is added to the plot.
+#' @param residuals Logical, if \code{TRUE}, a layer with partial residuals is
+#'   added to the plot. See vignette \href{https://cran.r-project.org/package=effects}{"Effect Displays with Partial Residuals"}
+#'   from \pkg{effects} for more details on partial residual plots.
+#' @param residuals.type The type of residuals to be plotted. Only applies if \code{residuals}
+#'   is \code{TRUE}. See \code{\link[stats:residuals.glm]{residuals.glm()}} for
+#'   details.
+#' @param residuals.line Logical, if \code{TRUE}, a loess-fit line is added to the
+#'   partial residuals plot. Only applies if \code{residuals} is \code{TRUE}.
 #' @param colors Character vector with color values in hex-format, valid
 #'   color value names (see \code{demo("colors")}) or a name of a
 #'   ggeffects-color-palette.
@@ -118,6 +126,9 @@ plot.ggeffects <- function(x,
                            ci.style = c("ribbon", "errorbar", "dash", "dot"),
                            facets,
                            add.data = FALSE,
+                           residuals = FALSE,
+                           residuals.type = NULL,
+                           residuals.line = FALSE,
                            colors = "Set1",
                            alpha = .15,
                            dodge = .25,
@@ -197,6 +208,41 @@ plot.ggeffects <- function(x,
   has_facets <- .obj_has_name(x, "facet") && length(unique(x$facet)) > 1
   has_panel <- .obj_has_name(x, "panel") && length(unique(x$panel)) > 1
 
+
+  # partial residuals?
+  if (residuals) {
+    obj_name <- attr(x, "model.name", exact = TRUE)
+    model <- NULL
+    if (!is.null(obj_name)) {
+      model <- tryCatch({
+        get(obj_name, envir = parent.frame())
+      }, error = function(e) {
+        NULL
+      })
+      if (is.null(model)) {
+        model <- tryCatch({
+          get(obj_name, envir = globalenv())
+        }, error = function(e) {
+          NULL
+        })
+      }
+    }
+
+    if (!is.null(model)) {
+      residual_data <- residualize_over_grid(grid = x, model = model, type = residuals.type)
+      attr(x, "residual_data") <- residual_data
+
+      ## TODO for now, we allow no continuous grouping varialbles for partial residuals
+      # it is difficult to match "raw data" values with the specific at-values
+      # for continuous variables
+
+      attr(x, "continuous.group") <- FALSE
+    } else {
+      warning("Could not find model object to extract residuals.", call. = FALSE)
+      residals <- FALSE
+    }
+  }
+
   # convert x back to numeric
   if (!is.numeric(x$x)) {
     if (x_is_factor && !.is_numeric_factor(x$x)) {
@@ -289,6 +335,8 @@ plot.ggeffects <- function(x,
         jitter = jitter,
         jitter.miss = jitter.miss,
         rawdata = rawdata,
+        residuals = residuals,
+        residuals.line = residuals.line,
         show.title = show.title,
         show.x.title = show.x.title,
         show.y.title = show.y.title,
@@ -335,6 +383,8 @@ plot.ggeffects <- function(x,
       jitter = jitter,
       jitter.miss = jitter.miss,
       rawdata = rawdata,
+      residuals = residuals,
+      residuals.line = residuals.line,
       show.title = show.title,
       show.x.title = show.x.title,
       show.y.title = show.y.title,
@@ -377,6 +427,8 @@ plot_panel <- function(x,
                        jitter,
                        jitter.miss,
                        rawdata,
+                       residuals,
+                       residuals.line,
                        show.title,
                        show.x.title,
                        show.y.title,
@@ -397,7 +449,8 @@ plot_panel <- function(x,
     x$group_col <- x$group
   }
 
-  # base plot, set mappings
+  # base plot, set mappings -----
+
   if (has_groups && !facets_grp && is_black_white && x_is_factor)
     p <- ggplot2::ggplot(x, ggplot2::aes_string(x = "x", y = "predicted", colour = "group_col", fill = "group_col", shape = "group"))
   else if (has_groups && !facets_grp && is_black_white && !x_is_factor)
@@ -410,117 +463,26 @@ plot_panel <- function(x,
     p <- ggplot2::ggplot(x, ggplot2::aes_string(x = "x", y = "predicted"))
 
 
-  # get color values
+  # get color values -----
+
   colors <- .get_colors(colors, length(unique(x$group)), isTRUE(attr(x, "continuous.group")))
 
 
-  # plot raw data points. we need an own aes for this
-  # we plot rawdata first, so it doesn't overlay the
-  # dots / lines for marginal effects
+  # plot raw data points -----
 
+  # get raw data
+  rawdat <- attr(x, "rawdata", exact = TRUE)
   if (rawdata) {
-    # get raw data and check, if any data available
-    rawdat <- attr(x, "rawdata", exact = TRUE)
-
-    if (!is.null(rawdat)) {
-      # make sure response is numeric
-      rawdat$response <- sjlabelled::as_numeric(rawdat$response)
-
-      # check if we have a group-variable with at least two groups
-      if (.obj_has_name(rawdat, "group")) {
-
-        if (isTRUE(attr(x, "continuous.group"))) {
-          rawdat$group_col <- as.numeric(as.character(rawdat$group))
-        } else {
-          rawdat$group_col <- rawdat$group
-        }
-
-        rawdat$group <- as.factor(rawdat$group)
-        # levels(rawdat$group) <- unique(x$group)
-        grps <- .n_distinct(rawdat$group) > 1
-      } else {
-        grps <- FALSE
-      }
-
-      # check if we have only selected values for groups, in this case
-      # filter raw data to match grouping colours
-      if (grps && isFALSE(attr(x, "continuous.group")) && .n_distinct(rawdat$group) > .n_distinct(x$group)) {
-        rawdat <- rawdat[which(rawdat$group %in% x$group), , drop = FALSE]
-      }
+    p <- .add_raw_data_to_plot(p, x, rawdat, ci.style, dot.alpha, dot.size, dodge, jitter, jitter.miss, colors)
+  }
 
 
-      # if we have groups, add colour aes, to map raw data to
-      # grouping variable
+  # plot partial residuals -----
 
-      if (grps)
-        mp <- ggplot2::aes_string(x = "x", y = "response", colour = "group_col")
-      else
-        mp <- ggplot2::aes_string(x = "x", y = "response")
-
-
-      # for binary response, no jittering by default
-
-      if ((attr(x, "logistic", exact = TRUE) == "1" && jitter.miss) || is.null(jitter)) {
-        p <- p + ggplot2::geom_point(
-          data = rawdat,
-          mapping = mp,
-          alpha = dot.alpha,
-          size = dot.size,
-          show.legend = FALSE,
-          inherit.aes = FALSE,
-          shape = 16
-        )
-      } else {
-        if (ci.style == "errorbar") {
-          if (grps) {
-            p <- p + ggplot2::geom_point(
-              data = rawdat,
-              mapping = ggplot2::aes_string(x = "x", y = "response", colour = "group_col"),
-              alpha = dot.alpha,
-              size = dot.size,
-              position = ggplot2::position_jitterdodge(
-                jitter.width = jitter[1],
-                jitter.height = jitter[2],
-                dodge.width = dodge
-              ),
-              show.legend = FALSE,
-              inherit.aes = FALSE,
-              shape = 16
-            )
-          } else {
-            p <- p + ggplot2::geom_point(
-              data = rawdat,
-              mapping = ggplot2::aes_string(x = "x", y = "response", fill = "group_col"),
-              alpha = dot.alpha,
-              size = dot.size,
-              position = ggplot2::position_jitterdodge(
-                jitter.width = jitter[1],
-                jitter.height = jitter[2],
-                dodge.width = dodge
-              ),
-              show.legend = FALSE,
-              inherit.aes = FALSE,
-              shape = 16,
-              color = colors[1]
-            )
-          }
-        } else {
-          p <- p + ggplot2::geom_jitter(
-            data = rawdat,
-            mapping = mp,
-            alpha = dot.alpha,
-            size = dot.size,
-            width = jitter[1],
-            height = jitter[2],
-            show.legend = FALSE,
-            inherit.aes = FALSE,
-            shape = 16
-          )
-        }
-      }
-    } else {
-      message("Raw data not available.")
-    }
+  # get residual data
+  residual_data <- attr(x, "residual_data", exact = TRUE)
+  if (isTRUE(residuals)) {
+    p <- .add_residuals_to_plot(p, x, residual_data, residuals.line, ci.style, line.size, dot.alpha, dot.size, dodge, jitter, colors)
   }
 
 
@@ -549,7 +511,8 @@ plot_panel <- function(x,
   }
 
 
-  # CI?
+  # CI ----
+
   if (ci) {
 
     # for a factor on x-axis, use error bars
@@ -630,7 +593,8 @@ plot_panel <- function(x,
   }
 
 
-  # facets?
+  # facets ----
+
   if (facets_grp) {
     # facet groups
     p <- p + ggplot2::facet_wrap(~group, scales = "free_x")
@@ -643,7 +607,8 @@ plot_panel <- function(x,
   }
 
 
-  # set colors
+  # set colors ----
+
   if (isTRUE(rawdata) && isTRUE(attr(x, "continuous.group"))) {
     p <- p +
       ggplot2::scale_color_gradientn(colors = colors, aesthetics = c("colour", "fill"), guide = "legend", breaks = as.numeric(levels(x$group)), limits = range(c(rawdat$group_col, x$group_col)))
@@ -653,13 +618,16 @@ plot_panel <- function(x,
   }
 
 
-  # show/hide titles
+  # show/hide titles ----
+
   if (!show.title) attr(x, "title") <- NULL
   if (!show.title) attr(x, "n.trials") <- NULL
   if (!show.x.title) attr(x, "x.title") <- NULL
   if (!show.y.title) attr(x, "y.title") <- NULL
 
-  # set axis titles
+
+  # set axis titles ----
+
   p <- p + ggplot2::labs(
     title = get_title(x, case),
     x = get_x_title(x, case),
@@ -675,7 +643,9 @@ plot_panel <- function(x,
       shape = get_legend_title(x, case)
     )
 
-  # no legend for fill-aes
+
+  # no legend for fill-aes ----
+
   p <- p + ggplot2::guides(fill = "none")
 
   if (is_black_white) {
@@ -685,7 +655,8 @@ plot_panel <- function(x,
   }
 
 
-  # show or hide legend?
+  # show or hide legend -----
+
   if (!show.legend) {
     p <- p + ggplot2::labs(
       colour = NULL,
@@ -695,7 +666,7 @@ plot_panel <- function(x,
   }
 
 
-  # for binomial family, fix coord
+  # for binomial family, fix coord ----
 
   if (attr(x, "logistic", exact = TRUE) == "1" && attr(x, "is.trial", exact = TRUE) == "0") {
     if (log.y) {
@@ -837,3 +808,234 @@ plot.ggalleffects <- function(x,
 .percents <- function(x) {
   insight::format_value(x = x, as_percent = TRUE, digits = 0)
 }
+
+
+
+
+
+.add_raw_data_to_plot <- function(p, x, rawdat, ci.style, dot.alpha, dot.size, dodge, jitter, jitter.miss, colors) {
+
+  if (!requireNamespace("ggplot2", quietly = FALSE)) {
+    stop("Package `ggplot2` needed to produce marginal effects plots. Please install it by typing `install.packages(\"ggplot2\", dependencies = TRUE)` into the console.", call. = FALSE)
+  }
+
+  # we need an own aes for this
+  # we plot rawdata first, so it doesn't overlay the
+  # dots / lines for marginal effects
+
+  if (!is.null(rawdat)) {
+    # make sure response is numeric
+    rawdat$response <- sjlabelled::as_numeric(rawdat$response)
+
+    # check if we have a group-variable with at least two groups
+    if (.obj_has_name(rawdat, "group")) {
+
+      if (isTRUE(attr(x, "continuous.group"))) {
+        rawdat$group_col <- as.numeric(as.character(rawdat$group))
+      } else {
+        rawdat$group_col <- rawdat$group
+      }
+
+      rawdat$group <- as.factor(rawdat$group)
+      # levels(rawdat$group) <- unique(x$group)
+      grps <- .n_distinct(rawdat$group) > 1
+    } else {
+      grps <- FALSE
+    }
+
+    # check if we have only selected values for groups, in this case
+    # filter raw data to match grouping colours
+    if (grps && isFALSE(attr(x, "continuous.group")) && .n_distinct(rawdat$group) > .n_distinct(x$group)) {
+      rawdat <- rawdat[which(rawdat$group %in% x$group), , drop = FALSE]
+    }
+
+
+    # if we have groups, add colour aes, to map raw data to
+    # grouping variable
+
+    if (grps)
+      mp <- ggplot2::aes_string(x = "x", y = "response", colour = "group_col")
+    else
+      mp <- ggplot2::aes_string(x = "x", y = "response")
+
+
+    # for binary response, no jittering by default
+
+    if ((attr(x, "logistic", exact = TRUE) == "1" && jitter.miss) || is.null(jitter)) {
+      p <- p + ggplot2::geom_point(
+        data = rawdat,
+        mapping = mp,
+        alpha = dot.alpha,
+        size = dot.size,
+        show.legend = FALSE,
+        inherit.aes = FALSE,
+        shape = 16
+      )
+    } else {
+
+      # no jitter
+      if (is.null(jitter)) {
+        jitter <- c(0, 0)
+      }
+
+      if (ci.style == "errorbar") {
+        if (grps) {
+          p <- p + ggplot2::geom_point(
+            data = rawdat,
+            mapping = ggplot2::aes_string(x = "x", y = "response", colour = "group_col"),
+            alpha = dot.alpha,
+            size = dot.size,
+            position = ggplot2::position_jitterdodge(
+              jitter.width = jitter[1],
+              jitter.height = jitter[2],
+              dodge.width = dodge
+            ),
+            show.legend = FALSE,
+            inherit.aes = FALSE,
+            shape = 16
+          )
+        } else {
+          p <- p + ggplot2::geom_point(
+            data = rawdat,
+            mapping = ggplot2::aes_string(x = "x", y = "response", fill = "group_col"),
+            alpha = dot.alpha,
+            size = dot.size,
+            position = ggplot2::position_jitterdodge(
+              jitter.width = jitter[1],
+              jitter.height = jitter[2],
+              dodge.width = dodge
+            ),
+            show.legend = FALSE,
+            inherit.aes = FALSE,
+            shape = 16,
+            color = colors[1]
+          )
+        }
+      } else {
+        p <- p + ggplot2::geom_jitter(
+          data = rawdat,
+          mapping = mp,
+          alpha = dot.alpha,
+          size = dot.size,
+          width = jitter[1],
+          height = jitter[2],
+          show.legend = FALSE,
+          inherit.aes = FALSE,
+          shape = 16
+        )
+      }
+    }
+  } else {
+    message("Raw data not available.")
+  }
+
+  p
+}
+
+
+
+
+#' @importFrom sjlabelled as_numeric
+.add_residuals_to_plot <- function(p, x, residuals, residuals.line, ci.style, line.size, dot.alpha, dot.size, dodge, jitter, colors) {
+  if (!requireNamespace("ggplot2", quietly = FALSE)) {
+    stop("Package `ggplot2` needed to produce marginal effects plots. Please install it by typing `install.packages(\"ggplot2\", dependencies = TRUE)` into the console.", call. = FALSE)
+  }
+
+  if (!is.null(residuals)) {
+
+    # make sure x on x-axis is on same scale
+    if (is.numeric(x$x) && !is.numeric(residuals$x)) {
+      residuals$x <- sjlabelled::as_numeric(residuals$x)
+    }
+
+    residuals$facet <- NULL
+    residuals$panel <- NULL
+
+
+    # check if we have a group-variable with at least two groups
+    if (.obj_has_name(residuals, "group")) {
+
+      if (isTRUE(attr(x, "continuous.group")) && is.numeric(x$group)) {
+        residuals$group_col <- as.numeric(as.character(residuals$group))
+      } else {
+        residuals$group_col <- as.factor(residuals$group)
+      }
+
+      residuals$group <- as.factor(residuals$group)
+      grps <- .n_distinct(residuals$group) > 1
+    } else {
+      grps <- FALSE
+    }
+
+    # check if we have only selected values for groups, in this case
+    # filter raw data to match grouping colours
+    if (grps && isFALSE(attr(x, "continuous.group")) && .n_distinct(residuals$group) > .n_distinct(x$group)) {
+      residuals <- residuals[which(residuals$group %in% x$group), , drop = FALSE]
+    }
+
+
+    # if we have groups, add colour aes, to map raw data to
+    # grouping variable
+
+    if (grps)
+      mp <- ggplot2::aes_string(x = "x", y = "predicted", colour = "group_col")
+    else
+      mp <- ggplot2::aes_string(x = "x", y = "predicted")
+
+
+
+    # if ("group" %in% colnames(residuals)) {
+    #   if (isTRUE(attr(x, "continuous.group"))) {
+    #     residuals$group_col <- as.numeric(as.character(residuals$group))
+    #   } else {
+    #     residuals$group_col <- residuals$group
+    #   }
+    #   residuals$group <- as.factor(residuals$group)
+    #   mp <- ggplot2::aes_string(x = "x", y = "predicted", colour = "group_col")
+    # } else {
+    #   mp <- ggplot2::aes_string(x = "x", y = "predicted")
+    # }
+
+
+
+    if (is.null(jitter)) {
+      p <- p + ggplot2::geom_point(
+        data = residuals,
+        mapping = mp,
+        alpha = dot.alpha,
+        size = dot.size,
+        show.legend = FALSE,
+        inherit.aes = FALSE,
+        shape = 16
+      )
+    } else {
+      p <- p + ggplot2::geom_jitter(
+        data = residuals,
+        mapping = mp,
+        alpha = dot.alpha,
+        size = dot.size,
+        width = jitter[1],
+        height = jitter[2],
+        show.legend = FALSE,
+        inherit.aes = FALSE,
+        shape = 16
+      )
+    }
+
+    if (isTRUE(residuals.line)) {
+      p <- p + ggplot2::geom_smooth(
+        data = residuals,
+        mapping = mp,
+        method = "loess",
+        inherit.aes = FALSE,
+        size = line.size,
+        se = FALSE
+      )
+    }
+
+  } else {
+    message("Partial residuals not available.")
+  }
+
+  p
+}
